﻿using System;

namespace producerConsumer
{
    /// <summary>
    /// Produces items and Adds them into the specified buffer
    /// </summary>
    public class Producer
    {
        private readonly IBuffer _buffer;
        private readonly int _howMany;
        /// <summary>
        /// The last element the producer add to the buffer.<br />
        /// When consumers sees this item, they know that no more elements will follow.
        /// </summary>
        public static readonly int LastElement = -1;

        /// <summary>
        /// Constructs a new object of this class.
        /// </summary>
        /// <param name="buffer">The buffer where the produced elements must be added to</param>
        /// <param name="howMany">The number of element this producer should produce</param>
        /// <exception cref="ArgumentNullException">if buffer is null</exception>
        /// <exception cref="ArgumentOutOfRangeException">if howMany &lt; 0</exception>
        public Producer(IBuffer buffer, int howMany)
        {
            if (buffer == null) { throw new ArgumentNullException("buffer"); }
            if (howMany < 0)
            {
                throw new ArgumentOutOfRangeException("howMany", howMany, "howMany must be a positive number");
            }
            _buffer = buffer;
            _howMany = howMany;
        }

        /// <summary>
        /// Runs this Producer<br />
        /// Instantiates the delegate type <code>ThreadStart</code>
        /// </summary>
        public void Run()
        {
            for (int i = 0; i < _howMany; i++)
            {
                _buffer.Add(i);
                Console.WriteLine("Producer added " + i);
            }
            _buffer.Add(LastElement);
        }
    }
}
