﻿using System;
using System.Threading;

namespace producerConsumer
{
    /// <summary>
    /// A buffer with unlimited capacity.
    /// </summary>
    public class UnboundedBuffer : AbstractBuffer
    {
        // http://stackoverflow.com/questions/1222184/do-i-need-to-lock-or-mark-as-volatile-when-accessing-a-simple-boolean-flag-in-c?rq=1
        private volatile Boolean _ending = false;

        public override int Take()
        {
            Monitor.Enter(Queue);
            try
            {
                if (_ending) { return Producer.LastElement; }
                while (IsEmpty())
                {
                    Monitor.Wait(Queue);
                }
                int element = Queue.Dequeue();
                if (element == Producer.LastElement) { _ending = true; }
                return element;
            }
            finally
            {
                Monitor.Exit(Queue);
            }
        }

        public override void Add(int element)
        {
            if (_ending)
            {
                throw new InvalidOperationException("Trying to Add after adding the LastItem");
            }
            Monitor.Enter(Queue);
            try
            {
                Queue.Enqueue(element);
                if (Count == 1)
                {
                    Monitor.PulseAll(Queue);
                }
            }
            finally
            {
                Monitor.Exit(Queue);
            }
        }
    }
}
