﻿using System;
using System.Threading;

namespace producerConsumer
{
    public class UnboundedBuffer : AbstractBuffer
    {
        private Boolean _ending = false;

        public override int Take()
        {
            Monitor.Enter(Queue);
            try
            {
                if (_ending) { return Producer.LastItem; }
                while (IsEmpty())
                {
                    Monitor.Wait(Queue);
                }
                int element = Queue.Dequeue();
                if (element == Producer.LastItem) { _ending = true; }
                return element;
            }
            finally
            {
                Monitor.Exit(Queue);
            }
        }

        public override void Put(int element)
        {
            if (_ending)
            {
                throw new InvalidOperationException("Trying to put after putting the STOP_VALUE");
            }
            Monitor.Enter(Queue);
            try
            {
                Queue.Enqueue(element);
                if (Count == 1)
                {
                    Monitor.PulseAll(Queue);
                }
            }
            finally
            {
                Monitor.Exit(Queue);
            }
        }
    }
}
